import { Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
import { AuthenticationModule } from '@modules/authentication/authentication.module';
import { EventEmitterModule } from '@nestjs/event-emitter';
import {
  MessageBrokerModule,
  MessageBrokerModuleOptions,
} from '@mobicoop/message-broker-module';
import { HealthModule, HealthModuleOptions } from '@mobicoop/health-module';
import { AuthorizationModule } from '@modules/authorization/authorization.module';
import {
  AUTHENTICATION_REPOSITORY,
  USERNAME_REPOSITORY,
} from '@modules/authentication/authentication.di-tokens';
import { MESSAGE_PUBLISHER } from './modules/message-publisher/message-publisher.di-tokens';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { ICheckRepository } from '@mobicoop/health-module/dist/core/domain/types/health.types';
import { MessagePublisherModule } from './modules/message-publisher/message-publisher.module';

@Module({
  imports: [
    ConfigModule.forRoot({ isGlobal: true }),
    EventEmitterModule.forRoot(),
    MessageBrokerModule.forRootAsync({
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: async (
        configService: ConfigService,
      ): Promise<MessageBrokerModuleOptions> => ({
        uri: configService.get<string>('MESSAGE_BROKER_URI'),
        exchange: configService.get<string>('MESSAGE_BROKER_EXCHANGE'),
        name: 'auth',
        handlers: {
          userUpdated: {
            routingKey: 'user.updated',
            queue: 'auth-user-updated',
          },
          userDeleted: {
            routingKey: 'user.deleted',
            queue: 'auth-user-deleted',
          },
        },
      }),
    }),
    HealthModule.forRootAsync({
      imports: [AuthenticationModule, MessagePublisherModule],
      inject: [
        AUTHENTICATION_REPOSITORY,
        USERNAME_REPOSITORY,
        MESSAGE_PUBLISHER,
      ],
      useFactory: async (
        authenticationRepository: ICheckRepository,
        usernameRepository: ICheckRepository,
        messagePublisher: MessagePublisherPort,
      ): Promise<HealthModuleOptions> => ({
        serviceName: 'auth',
        criticalLoggingKey: 'logging.auth.health.crit',
        checkRepositories: [authenticationRepository, usernameRepository],
        messagePublisher,
      }),
    }),
    AuthenticationModule,
    AuthorizationModule,
    MessagePublisherModule,
  ],
  controllers: [],
  exports: [AuthenticationModule, MessagePublisherModule],
})
export class AppModule {}
